@@ -1,4 +1,7 @@
+import io
+
 import os
+
 import sys
 import asyncio
 
@@ -12,7 +15,7 @@ import pandas as pd
 
 class GSDBS:
     def __init__(self, creadPath):
-        self.version = "0.1.23"
+        self.version = "0.1.25"
         self.credentials = None
         self.accessToken = None
         self.dtablename = None
@@ -26,6 +29,7 @@ class GSDBS:
         self.CGSResponseCode = {}
         self.setupCGSResponseCodes()
         self.debugModus = False
+        self.querybuffer = ''
         return
 
     def setDebugModus(self, debugModus):
@@ -57,7 +61,8 @@ class GSDBS:
                     'password': self.credentials['password']}
             access_token_response = requests.post(self.credentials['oauthurl'], data=data, verify=True,
                                                   allow_redirects=False,
-                                                  auth=(self.credentials['client_id'], self.credentials['client_secret']))
+                                                  auth=(
+                                                      self.credentials['client_id'], self.credentials['client_secret']))
             self.accessToken = json.loads(access_token_response.text)
             if 'error' in self.accessToken:
                 raise ValueError('error:' + self.accessToken['error'])
@@ -66,7 +71,7 @@ class GSDBS:
         return
 
     def setupCGSResponseCodes(self):
-        self.CGSResponseCode['0']  = 'SUCCESS'
+        self.CGSResponseCode['0'] = 'SUCCESS'
         self.CGSResponseCode['-1'] = 'MISSING (SUPER) DTABLE NAME'
         self.CGSResponseCode['-2'] = 'MISSING DATA LINK LIST'
         self.CGSResponseCode['-3'] = 'MISSING SRI BUILD INFO'
@@ -79,6 +84,8 @@ class GSDBS:
         self.CGSResponseCode['-10'] = 'STATEMENT ERROR'
         self.CGSResponseCode['-11'] = 'ERROR IN DATASCHEMA'
         self.CGSResponseCode['-12'] = 'TYPE ERROR'
+        self.CGSResponseCode['-13'] = 'Missing DTABLE'
+        self.CGSResponseCode['-14'] = 'Missing DTABLE SCHEMA'
 
     def setDTableName(self, dTableName, superDTableName="DTABLE"):
         if dTableName is None or superDTableName is None:
@@ -323,7 +330,6 @@ class GSDBS:
 
         return rc
 
-
     def addDObject(self, dtablename, sribuildinfo, dataframe, superdtablename="DTABLE", schemaCheck=True):
         self.superdtablename = superdtablename
         rc = self.setDTableName(dtablename)
@@ -382,8 +388,8 @@ class GSDBS:
 
         api_call_headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
         client = Client(transport=AIOHTTPTransport(url="https://glass-sphere-ai.de/gsgraphql",
-                        headers=api_call_headers, timeout=3600),
-                        fetch_schema_from_transport=False)
+                                                   headers=api_call_headers),
+                        headers=api_call_headers, timeout=3600)
 
         try:
             result = await client.execute_async(query)
@@ -402,7 +408,7 @@ class GSDBS:
 
         api_call_headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
         client = Client(transport=AIOHTTPTransport(url="https://glass-sphere-ai.de/gsgraphql",
-                        headers=api_call_headers, timeout=3600),
+                                                   headers=api_call_headers, timeout=3600),
                         fetch_schema_from_transport=False)
 
         try:
@@ -413,12 +419,106 @@ class GSDBS:
 
         return result
 
-    async def asyncDropDTable(self, dtablename):
-        statement = f"""mutation {{\n\tdropDTable(dtablename: {dtablename})\n}}"""
-        result = await self.asyncExecuteStatement(statement)
-        return result
+        async def asyncDropDTable(self, dtablename):
+            statement = f"""mutation {{\n\tdropDTable(dtablename: {dtablename})\n}}"""
+            result = await self.asyncExecuteStatement(statement)
+            return result
 
     def dropDTable(self, dtablename):
         statement = f"""mutation {{\n\tdropDTable(dtablename: {dtablename})\n}}"""
         result = self.executeStatement(statement)
         return result
+
+
+    def getDTableFullQuery(self, dtablename):
+        try:
+            self.querybuffer = 'query {\n'
+            self.querybuffer += dtablename + '\n'
+            self.getDTableQuery(dtablename)
+            self.querybuffer += '}'
+        except Exception as e:
+            return e
+        return self.querybuffer
+
+    def getDTableQuery(self, dtablename):
+        statement = f"""{{
+          __type(name: \"{dtablename}\") {{
+            name
+            fields {{
+              name
+              type {{
+                name
+                kind
+                ofType {{
+                  name
+                  kind
+                }}
+              }}
+            }}
+          }}
+        }}"""
+        result = self.executeStatement(statement)
+        if result == None:
+            raise ValueError('MISSING DTABLE')
+
+        schema = dict(result['__type'])
+        if schema == None:
+           raise ValueError('MISSING DTABLE SCHEMA')
+
+        schemaName = schema["name"]
+
+        self.querybuffer += '{\n'
+        for field in schema["fields"]:
+            if field['name'] == 'episodes':
+                continue
+            if field['name'] == 'dynamic_snapshot_timestamp':
+                continue
+            if field['name'] == 'totalrowcount':
+                continue
+            self.querybuffer += field['name'] + "\n"
+            if field['type'] != None:
+                if field['type']['ofType'] != None:
+                    self.getDTableQuery(field['type']['ofType']['name'])
+        self.querybuffer += '}\n'
+        return 0
+
+    def getForeignDTableName(self, dtablename, foreignLinkName):
+        statement = f"""{{
+          __type(name: \"{dtablename}\") {{
+            name
+            fields {{
+              name
+              type {{
+                name
+                kind
+                ofType {{
+                  name
+                  kind
+                }}
+              }}
+            }}
+          }}
+        }}"""
+        result = self.executeStatement(statement)
+        if result == None:
+            raise ValueError('MISSING DTABLE')
+        schema = dict(result['__type'])
+        if schema == None:
+           raise ValueError('MISSING DTABLE SCHEMA')
+        schemaName = schema["name"]
+
+        for field in schema["fields"]:
+            if field['name'] == foreignLinkName:
+                if field['type'] != None:
+                    if field['type']['ofType'] != None:
+                        return field['type']['ofType']['name']
+        return None
+
+    def getFileFromCloud(self, filename):
+        filename = filename.replace("/","$")
+        # storageURL = "https://glass-sphere-ai.de/storage/file/" + filename
+        storageURL = "http://localhost:8085/storage/file/" + filename
+        headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
+        resp = requests.get(storageURL, headers=headers)
+        buf = io.BytesIO(resp.content)
+        return buf
